#include "AngleParser.h"
#include "Constants.h"
#include <cmath>
#include <iostream>
#include <sstream>
#define THISCLASS AngleParser

THISCLASS::AngleParser(bool ns, bool we):
	mNumberIndex(0), mKeywordIndex(0), mState(0), mSign(0), mSignImplicit(1),
	mAngle(0), mAngleState(0), mNorthSouth(ns), mWestEast(we) {

}

THISCLASS::AngleParser(const std::string &str, bool ns, bool we):
	mNumberIndex(0), mKeywordIndex(0), mState(0), mSign(0), mSignImplicit(1),
	mAngle(0), mAngleState(0), mNorthSouth(ns), mWestEast(we) {

	ParseString(str);
}

bool THISCLASS::ParseString(const std::string &text) {
	int len = text.length();
	const char * buffer = text.c_str();
	//std::cerr << "Parsing string: " << text << std::endl;

	for (int i = 0; i <= len; i++) { // include the trailing 0 char
		char t = buffer[i];
		//std::cerr << "char " << i << ": " << (int)t << " " << t;
		//std::cerr << " (state " << mState << ", sign " << mSign << ", numberindex " << mNumberIndex << ", keywordindex" << mKeywordIndex << ")" << std::endl;
		if (mState == 0) {
			if (t == '-') {
				if (mSign == 0) {
					mSign = -1;
				} else {
					return false;
				}
			} else if ((t >= '0' && t <= '9') || t == '.') {
				if (mNumberIndex < mNumberMax) mNumber[mNumberIndex++] = t;
				mState = 1;
			} else if ((t >= 'A' && t <= 'Z') || (t >= 'a' && t <= 'z') || t == '\'' || t == '"' || t < 0) {
				if (mKeywordIndex < mKeywordMax) mKeyword[mKeywordIndex++] = t;
				mState = 2;
			} else if (t == ' ' || t == '\t' || t == '\n' || t == '\r' || t == 0) {
			} else {
				return false;
			}
		} else if (mState == 1) {
			if ((t >= '0' && t <= '9') || t == '.') {
				if (mNumberIndex < mNumberMax) mNumber[mNumberIndex++] = t;
			} else if ((t >= 'A' && t <= 'Z') || (t >= 'a' && t <= 'z') || t == '\'' || t == '"' || t < 0) {
				if (mKeywordIndex < mKeywordMax) mKeyword[mKeywordIndex++] = t;
				mState = 2;
			} else if ((t == ' ') || (t == '\t') || (t == '\n') || (t == '\r') || (t == 0)) {
				mState = 2;
			} else {
				return false;
			}
		} else if (mState == 2) {
			if ((t >= '0' && t <= '9') || t == '.') {
				if (! AddAngle()) return false;
				if (mNumberIndex < mNumberMax) mNumber[mNumberIndex++] = t;
				mState = 1;
			} else if ((t >= 'A' && t <= 'Z') || (t >= 'a' && t <= 'z') || t == '\'' || t == '"' || t < 0) {
				if (mKeywordIndex < mKeywordMax) mKeyword[mKeywordIndex++] = t;
			} else if (t == ' ' || t == '\t' || t == '\n' || t == '\r' || t == 0) {
				if (! AddAngle()) return false;
				mState = 0;
			} else {
				return false;
			}
		}
	}

	return true;
}

bool THISCLASS::AddAngle() {
	mNumber[mNumberIndex] = 0;
	mKeyword[mKeywordIndex] = 0;
	std::string number(mNumber);
	std::string keyword(mKeyword);
	mNumberIndex = 0;
	mKeywordIndex = 0;

	// Fix sign
	if (mSign == 0) mSign = 1;

	// Convert number
	double angle = 0;
	std::istringstream str(number);
	str >> angle;

	//std::cerr << "AddAngle: number '" << number << "'=" << angle << " keyword '" << keyword << "'" << std::endl;

	// Special case: uppercase "S" means "south", lowercase "s" means "seconds"
	if (keyword == "S") keyword = "south";

	// Keywords
	if (keyword == "south") {
		if (! mNorthSouth) return false;
		mSignImplicit = -1;
	} else if ((keyword == "north") || (keyword == "n")) {
		if (! mNorthSouth) return false;
		mSignImplicit = 1;
	} else if ((keyword == "west") || (keyword == "w")) {
		if (! mWestEast) return false;
		mSignImplicit = 1;
	} else if ((keyword == "east") || (keyword == "e")) {
		if (! mWestEast) return false;
		mSignImplicit = -1;
	} else if ((keyword == "d") || (keyword == "deg") || (keyword == "degree") || (keyword == "degrees")) {
		mAngleState = 0;
	} else if ((keyword == "m") || (keyword == "min") || (keyword == "minute") || (keyword == "minutes") || (keyword == "'")) {
		mAngleState = 1;
	} else if ((keyword == "s") || (keyword == "sec") || (keyword == "second") || (keyword == "seconds") || (keyword == "\"") || (keyword == "''")) {
		mAngleState = 2;
	}

	// Modify angle
	if (mAngleState == 0) {
		mAngle += angle / 180 * PI;
	} else if (mAngleState == 1) {
		mAngle += angle / 60 / 180 * PI;
	} else if (mAngleState == 2) {
		mAngle += angle / 3600 / 180 * PI;
	}

	mAngleState++;
	return true;
}

Angle THISCLASS::GetValue() {
	AddAngle();
	return Angle(mAngle * mSign * mSignImplicit);
}
