#ifndef classAngleParser
#define classAngleParser

#include <string>
#include "Angle.h"

class AngleParser;

/*!
	Parses an angle.

	If NorthSouth is enabled, the keywords "north", "south", "N" and "S" are allowed.
	If WestEast is enabled, the keywords "west", "east", "W" and "E" are allowed.

	The following strings are considered as equivalent:
		-34 56.33' 12.22''
		-3456.33'12.22''
		-34d56.33m12.22s
		-34d 56.33m 12.22s
		-56.33m 34d 12.22s
		-34 56.33 12.22
		34 deg 56.33 min 12.22 sec S
		34 56.33 12.22S
		34 degrees 56.33 minutes 12.22 seconds south
		34 56.33 12.22 s
		34d 56.33m 12.22s S
*/
class AngleParser {

private:
	static const int mNumberMax = 32;
	char mNumber[mNumberMax + 1];
	int mNumberIndex;
	static const int mKeywordMax = 32;
	char mKeyword[mKeywordMax + 1];
	int mKeywordIndex;
	int mState;
	int mSign;
	int mSignImplicit;
	double mAngle;
	int mAngleState;
	bool mNorthSouth;
	bool mWestEast;

	bool AddAngle();

public:
	//! Constructor.
	AngleParser(bool ns = false, bool we = false);
	//! Constructor.
	AngleParser(const std::string &str, bool ns = false, bool we = false);

	//! Parses a string and adds it to the angle.
	bool ParseString(const std::string &text);
	//! Returns the angle.
	Angle GetValue();
};

#endif
