#ifndef classApplication
#define classApplication

#include <string>
#include "BearingCalculator.h"
#include "AngleDisplay.h"
#include "DistanceDisplay.h"

class Application;

//! Application.
class Application {

protected:
	//! The BearingCalculator object.
	BearingCalculator mBearingCalculator;
	//! Whether the user wanted help.
	bool mHelpMode;
	//! The desired output format for angles.
	AngleDisplay::eFormat mAngleFormat;
	//! The desired output format for angles.
	DistanceDisplay::eFormat mDistanceFormat;

public:

	//! Constructor.
	Application();
	//! Destructor.
	~Application();

	//! Runs the application.
	int Run(int argc, char * argv[]);

	//! Sets an option.
	bool SetOption(const std::string &option, const std::string arg[], int argcc);
	//! Displays help.
	void Help();

	//! Displays the message for unknown options.
	bool UnknownOption(const std::string &option);
	//! Displays the message for too many arguments.
	bool TooManyArguments(const std::string &option);
	//! Checks if the number of arguments is in the allowed range.
	bool CheckArgumentCount(const std::string &option, int argcc, int min, int max);

	//! Displays the results.
	bool ShowResults();
};

#endif
