#include "BearingCalculator.h"
#include "Constants.h"
#define THISCLASS BearingCalculator

THISCLASS::BearingCalculator():
	mMyOrientation(1, 0, 0, 0, 1, 0, 0, 0, 1),
	mMyPosition(),
	mMyHeading(0),
	mMyPitch(0),
	mMyRoll(0),
	mTargetPosition(),
	mResultsAvailable(false),
	mHorizontalAngle(0),
	mVerticalAngle(0),
	mDistance(0) {

}

THISCLASS::~BearingCalculator() {
}

bool THISCLASS::Calculate() {
	// Get position vectors
	Vector3d vsrc = mMyPosition.GetPosition();
	Vector3d vdest = mTargetPosition.GetPosition();

	// Step 1: Latitude-Longitude Rotation Matrix
	//   X: towards north
	//   Y: towards east
	//   Z: towards the ground
	RotationMatrix rll;
	SphericVector src(vsrc);
	rll.SetSpheric(src.mLatitude, src.mLongitude);

	// Step 2: Gyro Rotation Matrix (with heading as yaw value)
	RotationMatrix rgyro;
	rgyro.SetGyro(mMyPitch, mMyRoll, mMyHeading);

	// Step 3: Antenna Mount Matrix
	//   X: horizontal 0 degrees angle
	//   Y: horizontal 90 degrees angle
	//   Z: perpendicular to the horizontal plane
	RotationMatrix rorientation = mMyOrientation;

	// Step 4: Calculate the combined rotation matrix and its inverse
	RotationMatrix r = rll * rgyro * rorientation;
	RotationMatrix rinv = r.Inverse();

	// Step 5: Calculate the distance vector
	Vector3d vdiff = vdest - vsrc;
	Vector3d tdiff = rinv * vdiff;

	// Step 6: Calculate the spheric parameters
	SphericVector stdiff(tdiff);

	// set results
	mDistance = stdiff.mRadius;
	mHorizontalAngle = stdiff.mLongitude;
	mVerticalAngle = stdiff.mLatitude;
	mResultsAvailable = true;
	return true;
}
