#ifndef classBearingCalculator
#define classBearingCalculator

#include "SphericVector.h"
#include "RotationMatrix.h"
#include "WGS84Position.h"
#include "Angle.h"
#include "Distance.h"

class BearingCalculator {

public:
	// Input fields
	//! Antenna Matrix
	RotationMatrix mMyOrientation;
	//! Our own position.
	WGS84Position mMyPosition;
	//! Our own heading.
	Angle mMyHeading;
	//! Our own pitch.
	Angle mMyPitch;
	//! Our own roll.
	Angle mMyRoll;
	//! Position of the target.
	WGS84Position mTargetPosition;

	// Result fields
	//! Set to true when the results are available.
	bool mResultsAvailable;
	//! Horizontal angle.
	Angle mHorizontalAngle;
	//! Vertical angle.
	Angle mVerticalAngle;
	//! The distance to the target.
	Distance mDistance;

	//! Constructor
	BearingCalculator();
	//! Destructor
	~BearingCalculator();

	//! Performs the calculation.
	bool Calculate();
};

#endif
