#include "DistanceParser.h"
#include "Constants.h"
#include <iostream>
#include <sstream>
#define THISCLASS DistanceParser

THISCLASS::DistanceParser():
	mNumberIndex(0), mKeywordIndex(0), mState(0), mSign(0), mDistance(0) {

}

THISCLASS::DistanceParser(const std::string &str):
	mNumberIndex(0), mKeywordIndex(0), mState(0), mSign(0), mDistance(0) {

	ParseString(str);
}

bool THISCLASS::ParseString(const std::string &text) {
	int len = text.length();
	const char * buffer = text.c_str();

	for (int i = 0; i <= len; i++) { // include the trailing 0 char
		char t = buffer[i];
		//printf("char %d: %d %c (state %d, sign %d, numberindex %d, keywordindex %d)\n", i, t, t, mState, mSign, mNumberIndex, mKeywordIndex);
		if (mState == 0) {
			if (t == '-') {
				if (mSign == 0) {
					mSign = -1;
				} else {
					return false;
				}
			} else if ((t >= '0' && t <= '9') || t == '.') {
				if (mNumberIndex < mNumberMax) mNumber[mNumberIndex++] = t;
				mState = 1;
			} else if ((t >= 'A' && t <= 'Z') || (t >= 'a' && t <= 'z') || t == '\'' || t == '"' || t < 0) {
				if (mKeywordIndex < mKeywordMax) mKeyword[mKeywordIndex++] = t;
				mState = 2;
			} else if ((t == ' ') || (t == '\t') || (t == '\n') || (t == '\r') || (t == 0)) {
			} else {
				return false;
			}
		} else if (mState == 1) {
			if ((t >= '0' && t <= '9') || (t == '.')) {
				if (mNumberIndex < mNumberMax) mNumber[mNumberIndex++] = t;
			} else if ((t >= 'A' && t <= 'Z') || (t >= 'a' && t <= 'z') || t == '\'' || t == '"' || t < 0) {
				if (mKeywordIndex < mKeywordMax) mKeyword[mKeywordIndex++] = t;
				mState = 2;
			} else if (t == ' ' || t == '\t' || t == '\n' || t == '\r' || t == 0) {
				mState = 2;
			} else {
				return false;
			}
		} else if (mState == 2) {
			if ((t >= '0' && t <= '9') || (t == '.')) {
				if (! AddDistance()) return false;
				if (mNumberIndex < mNumberMax) mNumber[mNumberIndex++] = t;
				mState = 1;
			} else if ((t >= 'A' && t <= 'Z') || (t >= 'a' && t <= 'z') || t == '\'' || t == '"' || t < 0) {
				if (mKeywordIndex < mKeywordMax) mKeyword[mKeywordIndex++] = t;
			} else if (t == ' ' || t == '\t' || t == '\n' || t == '\r' || t == 0) {
				if (! AddDistance()) return false;
				mState = 0;
			} else {
				return false;
			}
		}
	}

	return true;
}

bool THISCLASS::AddDistance() {
	mNumber[mNumberIndex] = 0;
	mKeyword[mKeywordIndex] = 0;
	std::string number(mNumber);
	std::string keyword(mKeyword);
	mNumberIndex = 0;
	mKeywordIndex = 0;

	// Fix sign
	if (mSign == 0) mSign = 1;

	// Convert number
	double distance = 0;
	std::istringstream str(number);
	str >> distance;

	// Keywords
	if (keyword == "" || keyword == "m" || keyword == "metre" || keyword == "meter" || keyword == "meters" || keyword == "metres") {
		mDistance += distance;
	} else if (keyword == "km" || keyword == "kilometer" || keyword == "kilometre" || keyword == "kilometers" || keyword == "kilometres") {
		mDistance += distance * 1000;
	} else if (keyword == "cm") {
		mDistance += distance / 100;
	} else if (keyword == "mm") {
		mDistance += distance / 1000;
	} else if (keyword == "''" || keyword == "in" || keyword == "inch" || keyword == "inches") {
		mDistance += distance * 0.0254;
	} else if (keyword == "'" || keyword == "ft" || keyword == "foot" || keyword == "feet") {
		mDistance += distance * 0.3048;
	} else if (keyword == "yd" || keyword == "yard" || keyword == "yards") {
		mDistance += distance * 0.9144;
	} else if (keyword == "mi" || keyword == "mile" || keyword == "miles") {
		mDistance += distance * 1609.344;
	} else if (keyword == "seamile" || keyword == "seamiles") {
		mDistance += distance * 1852;
	} else {
		return false;
	}

	return true;
}

Distance THISCLASS::GetValue() {
	AddDistance();
	return mDistance * mSign;
}

