#ifndef classRotationMatrix
#define classRotationMatrix

#include <math.h>
#include "Angle.h"
#include "Vector3d.h"

/*!
	 Fast rotation matrix calculation.
*/
class RotationMatrix {

public:
	double a[3][3];

	RotationMatrix();
	RotationMatrix(double a11, double a12, double a13, double a21, double a22, double a23, double a31, double a32, double a33);

	void SetIdentity();
	void SetVectors(const Vector3d &c1, const Vector3d &c2, const Vector3d &c3);
	void SetGyro(Angle pitch, Angle roll, Angle yaw);
	void SetSpheric(Angle longitude, Angle latitude);

	double SubDeterminant(int omitrow, int omitcol) const;
	double SubDeterminant(int row1, int row2, int col1, int col2) const;
	double Determinant() const {
		return 1;
	}
	double GeneralDeterminant() const;

	RotationMatrix GeneralInverse() const;
	RotationMatrix Inverse() const {
		return Transpose();
	}
	RotationMatrix Transpose() const;
	RotationMatrix Multiply(const RotationMatrix &m) const;

	Vector3d Multiply(const Vector3d &v) const;

	RotationMatrix operator * (const RotationMatrix &m) const {
		return Multiply(m);
	}
	Vector3d operator * (const Vector3d &v) const {
		return Multiply(v);
	}
};

#endif
