#ifndef classVector3d
#define classVector3d

#include <iostream>
#include <cmath>

//! 3d Vector (double).
class Vector3d {

public:
	double x, y, z;

	Vector3d(): x(0), y(0), z(0) {}
	Vector3d(double px, double py, double pz): x(px), y(py), z(pz) {}

	double Length() const {
		return sqrt(x * x + y * y + z * z);
	}
	double Length2() const {
		return x * x + y * y + z * z;
	}
	double Distance(const Vector3d &p) const {
		return sqrt((x - p.x) * (x - p.x) + (y - p.y) * (y - p.y) + (z - p.z) * (z - p.z));
	}
	double Distance2(const Vector3d &p) const {
		return (x - p.x) * (x - p.x) + (y - p.y) * (y - p.y) + (z - p.z) * (z - p.z);
	}

	Vector3d Floor() const {
		return Vector3d(floor(x), floor(y), floor(z));
	}
	Vector3d Ceil() const {
		return Vector3d(ceil(x), ceil(y), ceil(z));
	}
	Vector3d Round() const {
		return Vector3d(floor(x + 0.5), floor(y + 0.5), floor(z + 0.5));
	}
	Vector3d Move(const Vector3d &p) const {
		return Vector3d(x + p.x, y + p.y, z + p.z);
	}
	Vector3d Move(double sx, double sy, double sz) const {
		return Vector3d(x + sx, y + sy, z + sz);
	}
	Vector3d DotMultiply(const Vector3d &p) const {
		return Vector3d(x * p.x, y * p.y, z * p.z);
	}
	Vector3d DotMultiply(double sx, double sy, double sz) const {
		return Vector3d(x * sx, y * sy, z * sz);
	}
	Vector3d DotDivide(const Vector3d &p) const {
		return Vector3d(x / p.x, y / p.y, z / p.z);
	}
	Vector3d DotDivide(double sx, double sy, double sz) const {
		return Vector3d(x / sx, y / sy, z / sz);
	}
	Vector3d DotModulo(const Vector3d &p) const {
		return Vector3d(fmod(x, p.x), fmod(y, p.y), fmod(z, p.z));
	}
	Vector3d DotInv() const {
		return Vector3d(1 / x, 1 / y, 1 / z);
	}
	Vector3d Normalize() const;

	Vector3d operator - () const {
		return Vector3d(-x, -y, -z);
	}
	Vector3d operator + (const Vector3d &p) const {
		return Vector3d(x + p.x, y + p.y, z + p.z);
	}
	Vector3d operator - (const Vector3d &p) const {
		return Vector3d(x - p.x, y - p.y, z - p.z);
	}
	Vector3d operator * (double a) const {
		return Vector3d(x * a, y * a, z * a);
	}
	Vector3d operator / (double a) const {
		return Vector3d(x / a, y / a, z / a);
	}
	double operator * (const Vector3d &p) const {
		return x * p.x + y * p.y + z * p.z;
	}

	void operator += (const Vector3d &p) {
		x += p.x;
		y += p.y;
		z += p.z;
	}
	void operator -= (const Vector3d &p) {
		x -= p.x;
		y -= p.y;
		z -= p.z;
	}
	void operator *= (double a) {
		x *= a;
		y *= a;
		z *= a;
	}
	void operator /= (double a) {
		x /= a;
		y /= a;
		z /= a;
	}

	friend std::ostream &operator<<(std::ostream &stream, const Vector3d &p);
};

inline Vector3d Floor(const Vector3d &p) {
	return p.Floor();
}
inline Vector3d Ceil(const Vector3d &p) {
	return p.Ceil();
}
inline Vector3d Round(const Vector3d &p) {
	return p.Round();
}

#endif
